<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN">

<html lang="en">
<html xmlns="http://www.w3.org/1999/xhtml">

<head>
  {% block head %}
  <title>{% block title %}{% endblock %}</title>
  <link rel="stylesheet" href="style.css" />
  <script type="text/javascript" src="sorttable.js"></script>
  {% endblock %}
</head>

<body>
  <div class="content">{% block content %}{% endblock %}</div>
  <div class="footer">
    {% block footer %}
    Generated by <a href="{{ url }}">pyditz</a> version {{ version }}
    {% endblock %}
  </div>
</body>
