{% extends "base.html" %}

{% block title %}
Issues not assigned to any release
{% endblock %}

{% block head %}
{{ super() }}
{% endblock %}

{% block content %}
<div class="main">

<h1>{{ self.title() }}</h1>

{% from 'macros.html' import linkindex %}
{{ linkindex(project) }}

{% from 'macros.html' import issue_table %}
{{ issue_table(issues, show_component=show_components) }}

</div>
{% endblock %}
